import { Archive } from './asar';
import { Patch } from './patch';
type BackupOptions = {
    /**
     * Absolute path to the asar archive file to backup.
     */
    backupPath?: string;
};
type CreateBackupOptions = BackupOptions & {
    /**
     * Overwrite existing backup.
     *
     * Defaults to `false`.
     */
    overwrite?: boolean;
};
type RestoreBackupOptions = BackupOptions & {
    /**
     * Whether or not to remove the backup after restoration is complete.
     *
     * Defaults to `true`.
     */
    remove?: boolean;
};
export default class Asarmor {
    private readonly filePath;
    private archive;
    constructor(archivePath: string, archive: Archive);
    /**
     * Apply a patch to the asar archive.
     */
    patch(patch?: Patch): Archive;
    /**
     * Write modified asar archive to given absolute file path.
     */
    write(outputPath: string): Promise<string>;
    createBackup(options?: CreateBackupOptions): Promise<void>;
    restoreBackup(options?: RestoreBackupOptions): Promise<void>;
}
/**
 * Open and prepare asar archive file for modifications.
 */
export declare function open(asarFilePath: string): Promise<Asarmor>;
/**
 * Alias of `asarmor.write()`.
 */
export declare function close(asarmor: Asarmor, outputfilePath: string): Promise<string>;
export {};
